﻿using System;
using System.IO;

namespace TestAction
{
    /// <summary>
    /// 파일 로그 헬퍼
    /// </summary>
    public class FileLogHelper : ILogHelper 
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 동기 객체
        /// </summary>
        private object syncObject;

        /// <summary>
        /// 로그 루트 디렉토리 경로
        /// </summary>
        private string logRootDirectoryPath;

        /// <summary>
        /// 파일명
        /// </summary>
        private string fileName;

        /// <summary>
        /// 파일 경로
        /// </summary>
        private string filePath;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - FileLogHelper(logRootDirectoryPath, fileName)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="logRootDirectoryPath">로그 루트 디렉토리 경로</param>
        /// <param name="fileName">파일명</param>
        public FileLogHelper(string logRootDirectoryPath, string fileName)
        {
            this.syncObject = new object();

            this.logRootDirectoryPath = logRootDirectoryPath;
            this.fileName             = fileName;

            this.filePath = Path.Combine(this.logRootDirectoryPath, this.fileName);

            if(!Directory.Exists(this.logRootDirectoryPath))
            {
                Directory.CreateDirectory(this.logRootDirectoryPath);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 메시지 쓰기 - WriteMessage(message)

        /// <summary>
        /// 메시지 쓰기
        /// </summary>
        /// <param name="message">메시지</param>
        public void WriteMessage(string message)
        {
            lock(this.syncObject)
            {
                using(StreamWriter writer = File.AppendText(this.filePath))
                {
                    writer.WriteLine(message);
                }
            }
        }

        #endregion
        #region 로그 쓰기 - WriteLog(format, parameterArray)

        /// <summary>
        /// 로그 쓰기
        /// </summary>
        /// <param name="format">포맷 문자열</param>
        /// <param name="parameterArray">매개 변수 배열</param>
        public void WriteLog(string format, params object[] parameterArray)
        {
            string message;

            if(parameterArray.Length == 0)
            {
                message = format;
            }
            else
            {
                message = string.Format(format, parameterArray);
            }

            string log = string.Format("[{0}] {1}", DateTime.Now.ToString("HH:mm:ss"), message);

            WriteMessage(log);
        }

        #endregion
        #region 에러 로그 쓰기 - WriteErrorLog(exception, source)

        /// <summary>
        /// 에러 로그 쓰기
        /// </summary>
        /// <param name="exception">예외</param>
        /// <param name="source">소스 문자열</param>
        public void WriteErrorLog(Exception exception, string source)
        {
            lock(this.syncObject)
            {
                string title = string.Format("[{0}] {1}", DateTime.Now.ToString("HH:mm:ss"), source);

                using(StreamWriter writer = File.AppendText(this.filePath))
                {
                    writer.WriteLine(title);
                    writer.WriteLine("--------------------------------------------------");
                    writer.WriteLine(exception.ToString());
                    writer.WriteLine("--------------------------------------------------");
                }
            }
        }

        #endregion
        #region 에러 로그 쓰기 - WriteErrorLog(message, source)

        /// <summary>
        /// 에러 로그 쓰기
        /// </summary>
        /// <param name="message">예외 메시지</param>
        /// <param name="source">소스 문자열</param>
        public void WriteErrorLog(string message, string source)
        {
            lock (this.syncObject)
            {
                string title = string.Format("[{0}] {1}", DateTime.Now.ToString("HH:mm:ss"), source);

                using (StreamWriter writer = File.AppendText(this.filePath))
                {
                    writer.WriteLine(title);
                    writer.WriteLine("--------------------------------------------------");
                    writer.WriteLine(message);
                    writer.WriteLine("--------------------------------------------------");
                }
            }
        }

        #endregion
        #region 덤프 로그 쓰기 - WriteDumpLog(sourceObject, caption)

        /// <summary>
        /// 덤프 로그 쓰기
        /// </summary>
        /// <param name="sourceObject">예외</param>
        /// <param name="caption">제목 문자열</param>
        public void WriteDumpLog(object sourceObject, string caption)
        {
            lock(this.syncObject)
            {
                string title = string.Format("[{0}] {1}", DateTime.Now.ToString("HH:mm:ss"), caption);

                using(StreamWriter writer = File.AppendText(this.filePath))
                {
                    writer.WriteLine(title);
                    writer.WriteLine("--------------------------------------------------");
                    writer.WriteLine(sourceObject.ToString());
                    writer.WriteLine("--------------------------------------------------");
                }
            }
        }

        #endregion
    }
}