﻿using System;
using System.Threading;

namespace TestLibrary
{
    /// <summary>
    /// 반복 작업자
    /// </summary>
    public class RepeatWorker : IDisposable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region Field

        /// <summary>
        /// 로그 헬퍼
        /// </summary>
        protected ILogHelper logHelper = null;

        /// <summary>
        /// 스레드
        /// </summary>
        protected Thread thread = null;

        /// <summary>
        /// 루프 계속 여부
        /// </summary>
        protected bool continueLoop = true;

        /// <summary>
        /// 휴지 여부
        /// </summary>
        protected bool isSleep = false;

        /// <summary>
        /// 휴지 시간 (단위 : 밀리초)
        /// </summary>
        protected int sleepTime;

        /// <summary>
        /// 작업 액션
        /// </summary>
        protected Action<object> workAction;

        /// <summary>
        /// 작업 매개 변수
        /// </summary>
        protected object workParameter;

        /// <summary>
        /// 실행 여부
        /// </summary>
        protected bool isRunning = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 로그 헬퍼 - LogHelper

        /// <summary>
        /// 로그 헬퍼
        /// </summary>
        public ILogHelper LogHelper
        {
            get
            {
                return this.logHelper;
            }
            set
            {
                this.logHelper = value;
            }
        }

        #endregion
        #region 스레드 - Thread

        /// <summary>
        /// 스레드
        /// </summary>
        public Thread Thread
        {
            get
            {
            return this.thread;
            }
        }
        
        #endregion
        #region 휴지 시간 (단위 : 밀리초) - SleepTime

        /// <summary>
        /// 휴지 시간 (단위 : 밀리초)
        /// </summary>
        public int SleepTime
        {
            get
            {
                return this.sleepTime;
            }
            set
            {
                this.sleepTime = value;
            }
        }

        #endregion
        #region 작업 액션 - WorkAction

        /// <summary>
        /// 작업 액션
        /// </summary>
        public Action<object> WorkAction
        {
            get
            {
                return this.workAction;
            }
            set
            {
                this.workAction = value;
            }
        }

        #endregion
        #region 작업 매개 변수 - WorkParameter

        /// <summary>
        /// 작업 매개 변수
        /// </summary>
        public object WorkParameter
        {
            get
            {
                return this.workParameter;
            }
            set
            {
                this.workParameter = value;
            }
        }

        #endregion
        #region 실행 여부 - IsRunning

        /// <summary>
        /// 실행 여부
        /// </summary>
        public bool IsRunning
        {
            get
            {
                return this.isRunning;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - RepeatWorker(workAction, workParameter, sleepTime)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="workAction">작업 액션</param>
        /// <param name="workParameter">작업 매개 변수</param>
        /// <param name="sleepTime">휴지 시간 (단위 : 밀리초(</param>
        public RepeatWorker(Action<object> workAction, object workParameter, int sleepTime = 1000)
        {
            this.workAction    = workAction;
            this.workParameter = workParameter;
            this.sleepTime     = sleepTime;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 시작하기 - Start()

        /// <summary>
        /// 시작하기
        /// </summary>
        public void Start()
        {
            try
            {
                if(this.isRunning)
                {
                    return;
                }

                this.isRunning = true;

                #region 스레드를 설정한다.

                if(this.thread != null)
                {
                    if(this.thread.IsAlive)
                    {
                        this.thread.Abort();
                    }

                    this.thread = null;
                }

                this.thread = new Thread(new ThreadStart(ProcessThread));

                this.thread.IsBackground = true;

                #endregion

                this.thread.Start();
            }
            catch(Exception exception)
            {
                this.isRunning = false;

                this.logHelper?.WriteErrorLog(exception, "ERROR START FUNCTION");

                throw exception;
            }
        }

        #endregion
        #region 중단하기 - Stop()

        /// <summary>
        /// 중단하기
        /// </summary>
        public void Stop()
        {
            try
            {
                if(!this.isRunning)
                {
                    return;
                }

                this.isRunning = false;

                #region 스레드를 중단한다.

                this.continueLoop = false;

                Thread.Sleep(500);

                if(this.thread != null && this.thread.IsAlive)
                {
                    if(this.isSleep)
                    {
                        this.thread.Abort();
                    }
                }

                #endregion
            }
            catch(Exception exception)
            {
                this.logHelper?.WriteErrorLog(exception, "ERROR STOP FUNCTION");
            }
        }

        #endregion
        #region 리소스 해제하기 - Dispose()

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        public void Dispose()
        {
            Stop();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 스레드 처리하기 - ProcessThread()

        /// <summary>
        /// 스레드 처리하기
        /// </summary>
        private void ProcessThread()
        {
            while(this.continueLoop)
            {
                try
                {
                    this.workAction?.Invoke(this.workParameter);
                }
                catch(Exception exception)
                {
                    this.logHelper?.WriteErrorLog(exception, "ERROR WHEN PROCESS THREAD");
                }

                if(!this.continueLoop)
                {
                    break;
                }

                this.isSleep = true;

                Thread.Sleep(this.sleepTime);

                this.isSleep = false;
            }
        }

        #endregion
    }
}