﻿using System;
using System.Diagnostics;
using System.IO;
using System.Reflection;

namespace TestLibrary
{
    /// <summary>
    /// 테스트 노드
    /// </summary>
    public class TestNode
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        #region 로그

        /// <summary>
        /// 로그 헬퍼
        /// </summary>
        private ILogHelper logHelper;

        #endregion
        #region 실행 작업자

        /// <summary>
        /// 실행 작업자 주기 (단위 : 밀리초)
        /// </summary>
        private int executeWorkerInterval = 1000; // 0.1초

        /// <summary>
        /// 실행 작업자
        /// </summary>
        private RepeatWorker executeWorker = null;

        #endregion

        /// <summary>
        /// 실행 여부
        /// </summary>
        private bool isRunning = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 실행 여부 - IsRunning

        /// <summary>
        /// 실행 여부
        /// </summary>
        public bool IsRunning
        {
            get
            {
                return this.isRunning;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - TestNode()

        /// <summary>
        /// 생성자
        /// </summary>
        public TestNode()
        {
            #region 로그 헬퍼를 설정한다.

            this.logHelper = new FileLogHelper("d:\\", "TestNode.log");

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 시작하기 - Start()

        /// <summary>
        /// 시작하기
        /// </summary>
        public void Start()
        {
            try
            {
                this.logHelper?.WriteLog("BEGIN START FUNCTION");

                if(this.isRunning)
                {
                    this.logHelper?.WriteLog("STOP START FUNCTION : AlreadyRunning");

                    return;
                }

                this.isRunning = true;

                #region 실행 작업자를 설정한다.

                if(this.executeWorker != null)
                {
                    if(this.executeWorker.IsRunning)
                    {
                        this.executeWorker.Stop();
                    }

                    this.executeWorker = null;
                }

                this.executeWorker = new RepeatWorker(new Action<object>(ProcessExecute), null, this.executeWorkerInterval);

                #endregion

                this.executeWorker.Start();

                this.logHelper?.WriteLog("END START FUNCTION");
            }
            catch(Exception exception)
            {
                this.isRunning = false;

                this.logHelper?.WriteErrorLog(exception, "ERROR START FUNCTION");

                throw exception;
            }
        }

        #endregion
        #region 중단하기 - Stop()

        /// <summary>
        /// 중단하기
        /// </summary>
        public void Stop()
        {
            try
            {
                this.logHelper?.WriteLog("BEGIN STOP FUNCTION");

                if(!this.isRunning)
                {
                    this.logHelper?.WriteLog("STOP STOP FUNCTION : AlreadyStopped");

                    return;
                }

                this.isRunning = false;

                #region 실행 작업자를 중단한다.

                this.executeWorker.Stop();

                this.executeWorker = null;

                #endregion

                this.logHelper?.WriteLog("END STOP FUNCTION");
            }
            catch(Exception exception)
            {
                this.isRunning = false;

                this.logHelper?.WriteErrorLog(exception, "ERROR STOP FUNCTION");
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 실행 처리하기 - ProcessExecute(parameter)

        /// <summary>
        /// 실행 처리하기
        /// </summary>
        /// <param name="parameter">매개 변수</param>
        private void ProcessExecute(object parameter)
        {
            try
            {
                this.logHelper?.WriteLog("테스트 메시지");
            }
            catch(Exception exception)
            {
                this.logHelper?.WriteErrorLog(exception, "ERROR PROCESS EXECUTE FUNCTION");
            }
        }

        #endregion
    }
}